# -*- encoding: utf-8 -*-

from django.forms import ModelMultipleChoiceField
from django.utils.encoding import force_text
from sortedmanytomany.widgets import SortedFilteredSelectMultiple


class SortedModelMultipleChoiceField(ModelMultipleChoiceField):
    def __init__(self, queryset, required=True, widget=None, label=None, initial=None, help_text='', *args, **kwargs):
        super(ModelMultipleChoiceField, self).__init__(queryset, None, required, widget, label, initial, help_text, *args, **kwargs)
        self.widget = SortedFilteredSelectMultiple(verbose_name=queryset.model._meta.verbose_name_plural)

    def has_changed(self, initial, data):
        if initial is None:
            initial = []
        if data is None:
            data = []
        if len(initial) != len(data):
            return True
        initial_list = [force_text(value) for value in self.prepare_value(initial)]
        data_list = [force_text(value) for value in data]
        return data_list != initial_list

    def _check_values(self, value):
        qs = super(SortedModelMultipleChoiceField, self)._check_values(value)
        ordering = 'CASE %s END' % ' '.join(['WHEN id=%s THEN %s' % (pk, i) for i, pk in enumerate(value)])
        return qs.extra(select={'ordering': ordering}, order_by=('ordering',))



# -*- encoding: utf-8 -*-

from django.db import router, transaction
from django.db.models import ManyToManyField, signals
from django.db.models.fields.related_descriptors import create_forward_many_to_many_manager, ManyToManyDescriptor
from django.utils.functional import cached_property
from sortedmanytomany.forms import SortedModelMultipleChoiceField


def create_sorted_forward_many_to_many_manager(superclass, rel, *args, **kwargs):
    RelatedManager = create_forward_many_to_many_manager(superclass, rel, *args, **kwargs)
    print 'create_sorted_forward_many_to_many_manager'

    class ManyRelatedManager(RelatedManager):
        def __call__(self, **kwargs):
            manager = getattr(self.model, kwargs.pop('manager'))
            manager_class = create_sorted_forward_many_to_many_manager(manager.__class__, rel, reverse)
            return manager_class(instance=self.instance)
        do_not_call_in_templates = True

        def get_queryset(self):
            qs = super(ManyRelatedManager, self).get_queryset()
            return qs.extra(order_by=['%s.%s' % (rel.through._meta.db_table, 'id', )])

        get_query_set = get_queryset

        def set(self, objs, **kwargs):
            # Choosing to clear first will ensure the order is maintained.
            kwargs['clear'] = True
            super(ManyRelatedManager, self).set(objs, **kwargs)
        set.alters_data = True

        def _add_items(self, source_field_name, target_field_name, *objs):
            new_ids = [obj.id for obj in objs]
            db = router.db_for_write(self.through, instance=self.instance)

            with transaction.atomic(using=db, savepoint=False):
                if self.reverse or source_field_name == self.source_field_name:
                    signals.m2m_changed.send(sender=self.through, action='pre_add', instance=self.instance, reverse=self.reverse, model=self.model, pk_set=new_ids, using=db)

                self.through._default_manager.using(db).bulk_create([
                    self.through(**{
                        '%s_id' % source_field_name: self.related_val[0],
                        '%s_id' % target_field_name: obj_id,
                    })
                    for obj_id in new_ids
                ])

                if self.reverse or source_field_name == self.source_field_name:
                    signals.m2m_changed.send(sender=self.through, action='post_add', instance=self.instance, reverse=self.reverse, model=self.model, pk_set=new_ids, using=db)

    return ManyRelatedManager


class SortedManyToManyDescriptor(ManyToManyDescriptor):
    def __init__(self, field):
        super(SortedManyToManyDescriptor, self).__init__(field.remote_field)

    @cached_property
    def related_manager_cls(self):
        model = self.rel.related_model if self.reverse else self.rel.model
        return create_sorted_forward_many_to_many_manager(model._default_manager.__class__, self.rel, reverse=self.reverse,)


class SortedManyToManyField(ManyToManyField):
    def contribute_to_class(self, cls, name, **kwargs):
        super(SortedManyToManyField, self).contribute_to_class(cls, name, **kwargs)
        # Add the descriptor for the m2m relation
        setattr(cls, self.name, SortedManyToManyDescriptor(self))

    def formfield(self, **kwargs):
        defaults = {}
        defaults['form_class'] = SortedModelMultipleChoiceField
        defaults.update(kwargs)
        return super(SortedManyToManyField, self).formfield(**defaults)



# -*- encoding: utf-8 -*-

from django.contrib.admin.widgets import FilteredSelectMultiple
from django.utils.encoding import force_text
from django.utils.html import escapejs
from django.utils.safestring import mark_safe


class SortedFilteredSelectMultiple(FilteredSelectMultiple):
    def __init__(self, verbose_name, attrs=None, choices=()):
        self.verbose_name = verbose_name
        super(FilteredSelectMultiple, self).__init__(attrs, choices)

    def render(self, name, value, attrs=None, choices=()):
        if attrs is None:
            attrs = {}
        attrs['class'] = 'selectfilter'
        output = [super(FilteredSelectMultiple, self).render(name, value, attrs, choices)]
        output.append('<script type="text/javascript">addEvent(window, "load", function(e) { SortedSelectFilter.init("id_%s", "%s", 0); });</script>\n' % (name, escapejs(self.verbose_name)))
        return mark_safe(''.join(output))

    def render_option(self, selected_choices, option_value, option_label):
        if option_value is None:
            option_value = ''
        option_value = force_text(option_value)
        if option_value in selected_choices:
            selected_html = mark_safe(' selected="selected"')
            order_html = mark_safe(' data-order="%i"' % self.selected_ordered_choices.index(option_value))
        else:
            selected_html = ''
            order_html = ''
        return '<option value="%s"%s%s>%s</option>' % (option_value, order_html, selected_html, option_label)

    def render_options(self, choices, selected_choices):
        self.selected_ordered_choices = [str(c) for c in selected_choices]
        return super(SortedFilteredSelectMultiple, self).render_options(choices, selected_choices)

    class Media:
        css = {'all': ('sortedmanytomany/css/widget.css',)}
        js = ('admin/js/core.js', 'sortedmanytomany/js/SortedSelectBox.js', 'sortedmanytomany/js/SortedSelectFilter.js')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from os.path import dirname, abspath, join
from setuptools import find_packages, setup

URL = 'https://github.com/cereigido/django-sortedmanytomany/'
VERSION = __import__('sortedmanytomany').__version__

with open(abspath(join(dirname(__file__), 'README.md'))) as fileobj:
    README = fileobj.read().strip()

setup(
    name='django-sortedmanytomany',
    packages=find_packages(),
    version=VERSION,
    author='Paulo Cereigido',
    author_email='paulocereigido@gmail.com',
    description=('An extension of Django ManyToMany with ordering'),
    long_description=README,
    url=URL,
    download_url='%s/tarball/%s' % (URL, VERSION),
    zip_safe=False,
    include_package_data=True,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Database',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Internet :: WWW/HTTP :: Site Management',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    install_requires=[
        'Django>=1.9',
    ],
)
